
from .abstract_channel import AbstractChannel

from builtins import int

try:
    basestring
except NameError:
    basestring = str


class CustomerIO(AbstractChannel):

    timeout = 10

    def __init__(self, campaign_id, data, additional_recipient_emails=[]):
        self.set_campaign_id(campaign_id)
        self.set_additional_recipient_emails(additional_recipient_emails)
        self.set_data(data)

    def set_campaign_id(self, campaign_id):
        if not isinstance(campaign_id, int):
            raise ValueError('campaign id have to be an integer')
        self.campaign_id = campaign_id

    def set_data(self, data):
        if not isinstance(data, dict):
            raise ValueError('data have to be a dicionary')
        self.data = data

    def set_data_key(self, key, value):
        if not isinstance(key, basestring):
            raise ValueError('key have to be a string')
        self.data[key] = value

    def set_additional_recipient_emails(self, additional_recipient_emails):
        if not isinstance(additional_recipient_emails, (list,)):
            additional_recipient_emails = [additional_recipient_emails]
        self.additional_recipient_emails = additional_recipient_emails

    def json(self):
        payload = dict()
        payload['campaignId'] = self.campaign_id
        payload['data'] = self.data
        payload['additionalEmails'] = self.additional_recipient_emails
        return payload

    def attachments(self):
        return []

    def notif_type(self):
        return 'CUSTOMERIO_NOTIFICATION'



from .abstract_channel import AbstractChannel


class WhatsappCustomMessageSender(AbstractChannel):

    def __init__(self, phone_number, message):
        self.phone_number = phone_number
        self.message = message

    def notif_type(self):
        return 'SEND_CUSTOM_MESSAGE'

    def json(self):
        return {
            'phone_number': self.phone_number,
            'message': self.message
        }

    def attachments(self):
        return []



from .abstract_channel import AbstractChannel

try:
    basestring
except NameError:
    basestring = str

class CreateGroupWhatsapp(AbstractChannel):

    def __init__(self, subject):
        self.set_subject(subject)

    def set_subject(self, subject):
        if not isinstance(subject, basestring):
            raise ValueError('subject have to be string')
        self.subject = subject

    def json(self):
        payload = dict()
        payload['subject'] = self.subject
        return payload

    def notif_type(self):
        return 'CREATE_GROUP_WHATSAPP'

    def attachments(self):
        return []



from .abstract_channel import AbstractChannel


class ProjectPushNotification(AbstractChannel):

    def __init__(self, project_key, event_type, title, body, data, url=None):
        self.set_project_key(project_key)
        self.set_event_type(event_type)
        self.set_title(title)
        self.set_body(body)
        self.set_data(data)
        if url is not None:
            self.set_url(url)

    def set_project_key(self, project_key):
        self.project_key = project_key

    def set_event_type(self, event_type):
        self.event_type = event_type

    def set_title(self, title):
        self.title = title

    def set_body(self, body):
        self.body = body

    def set_data(self, data):
        self.data = data

    def set_url(self, url):
        self.url = url

    def json(self):
        payload = dict()
        payload['projectKey'] = self.project_key
        payload['eventType'] = self.event_type
        payload['title'] = self.title
        payload['body'] = self.body
        payload['data'] = self.data
        if hasattr(self, 'url'):
            payload['url'] = self.url
        return payload

    def notif_type(self):
        return 'PROJECT_PUSH_NOTIFICATION'

    def attachments(self):
        return []



from .abstract_channel import AbstractChannel

class ListGroupWhatsapp(AbstractChannel):

    def json(self):
        payload = dict()
        return payload

    def notif_type(self):
        return 'LIST_GROUP_WHATSAPP'

    def attachments(self):
        return []


from django.test import TestCase
from .email import Email


class EmailTest(TestCase):

    def test_initialization_with_string_recipient(self):
        email = Email('some subject', 'some sender', 'some recipient')
        self.assertEqual(email.subject, 'some subject')
        self.assertEqual(email.sender, 'some sender')
        self.assertEqual(email.recipients, ['some recipient'])

    def test_initialization_with_list_recipient(self):
        email = Email('some subject', 'some sender', ['r1', 'r2', 'r3'])
        self.assertEqual(email.subject, 'some subject')
        self.assertEqual(email.sender, 'some sender')
        self.assertEqual(email.recipients, ['r1', 'r2', 'r3'])

    def test_set_recipient_with_list(self):
        email = Email('S1', 'SS', [])
        email.set_recipients(['x1', 'x2', 'x3'])
        self.assertEqual(email.recipients, ['x1', 'x2', 'x3'])

    def test_set_recipient_with_string(self):
        email = Email('S1', 'SS', [])
        email.set_recipients('somerecipient')
        self.assertEqual(email.recipients, ['somerecipient'])

    def test_add_recipient_with_list(self):
        email = Email('S1', 'SS', [])
        email.add_recipients(['y1', 'y2', 'y3'])
        email.add_recipients(['y4', 'y5'])
        email.add_recipients(['y6', 'y7', 'y8', 'y9'])
        self.assertEqual(email.recipients, ['y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8', 'y9'])

    def test_add_recipient_with_string(self):
        email = Email('S1', 'SS', [])
        email.add_recipients('y1')
        email.add_recipients('y2')
        email.add_recipients('y3')
        self.assertEqual(email.recipients, ['y1', 'y2', 'y3'])

    def test_set_recipient_cc_with_list(self):
        email = Email('S1', 'SS', [])
        email.set_recipients_cc(['x1', 'x2', 'x3'])
        self.assertEqual(email.recipients_cc, ['x1', 'x2', 'x3'])

    def test_set_recipient_cc_with_string(self):
        email = Email('S1', 'SS', [])
        email.set_recipients_cc('somerecipient')
        self.assertEqual(email.recipients_cc, ['somerecipient'])

    def test_add_recipient_cc_with_list(self):
        email = Email('S1', 'SS', [])
        email.add_recipients_cc(['y1', 'y2', 'y3'])
        email.add_recipients_cc(['y4', 'y5'])
        email.add_recipients_cc(['y6', 'y7', 'y8', 'y9'])
        self.assertEqual(email.recipients_cc, ['y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8', 'y9'])

    def test_add_recipient_cc_with_string(self):
        email = Email('S1', 'SS', [])
        email.add_recipients_cc('y1')
        email.add_recipients_cc('y2')
        email.add_recipients_cc('y3')
        self.assertEqual(email.recipients_cc, ['y1', 'y2', 'y3'])

    def test_set_recipient_bcc_with_list(self):
        email = Email('S1', 'SS', [])
        email.set_recipients_bcc(['x1', 'x2', 'x3'])
        self.assertEqual(email.recipients_bcc, ['x1', 'x2', 'x3'])

    def test_set_recipient_bcc_with_string(self):
        email = Email('S1', 'SS', [])
        email.set_recipients_bcc('somerecipient')
        self.assertEqual(email.recipients_bcc, ['somerecipient'])

    def test_add_recipient_bcc_with_list(self):
        email = Email('S1', 'SS', [])
        email.add_recipients_bcc(['y1', 'y2', 'y3'])
        email.add_recipients_bcc(['y4', 'y5'])
        email.add_recipients_bcc(['y6', 'y7', 'y8', 'y9'])
        self.assertEqual(email.recipients_bcc, ['y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8', 'y9'])

    def test_add_recipient_bcc_with_string(self):
        email = Email('S1', 'SS', [])
        email.add_recipients_bcc('y1')
        email.add_recipients_bcc('y2')
        email.add_recipients_bcc('y3')
        self.assertEqual(email.recipients_bcc, ['y1', 'y2', 'y3'])

    def test_add_attachment_with_list(self):
        email = Email('S1', 'SS', [])
        email.add_attachment('f1', 'c1', 'ct1')
        email.add_attachment('f2', 'c2', 'ct2')
        email.add_attachment('f3', 'c3', 'ct3')
        self.assertItemsEqual(email.email_attachments, [
            ('f1', 'c1', 'ct1'),
            ('f2', 'c2', 'ct2'),
            ('f3', 'c3', 'ct3')
        ])




from notif.channels.abstract_channel import AbstractChannel
from notif.channels.project_push_notification import ProjectPushNotification
from notif.channels.user_push_notification import UserPushNotification
from notif.channels.email import Email
from notif.channels.customerio import CustomerIO
from notif.channels.create_group_whatsapp import CreateGroupWhatsapp
from notif.channels.list_group_whatsapp import ListGroupWhatsapp
from notif.channels.whatsapp_template_message_sender import WhatsappTemplateMessageSender
from notif.channels.whatsapp_template_message_sender import WhatsappTemplateMessageSenderV3
from notif.channels.whatsapp_custom_message_sender import WhatsappCustomMessageSender
from notif.channels.sms_message_sender import SMSMessageSender

__all__ = ['AbstractChannel', 'ProjectPushNotification', 'UserPushNotification', 'Email', 'CustomerIO', 'CreateGroupWhatsapp', 'ListGroupWhatsapp', 'WhatsappTemplateMessageSender', 'WhatsappCustomMessageSender', 'WhatsappTemplateMessageSenderV3', 'SMSMessageSender']




from builtins import object
import json
import requests

from .constants import K_NOTIF_API_KEY
from .constants import K_NOTIF_API_URL


class NotifApi(object):

    def __init__(self, api_key=K_NOTIF_API_KEY):
        self.notif_service_url = K_NOTIF_API_URL
        self.api_key = api_key

    def bulk_send(self, notifications, is_async=False):
        notification_body_list = []
        attachments = []
        for index, notif in enumerate(notifications):
            notification_body_list += [{
                'data': notif.json(),
                'type': notif.notif_type(),
                'is_async': is_async,
            }]
            for attachment in notif.attachments():
                attachments += [('attachment_%d' % index, attachment)]
        body = json.dumps(notification_body_list)

        if len(attachments) > 0:
            response = requests.post(
                self.notif_service_url,
                data={'body': body},
                headers={'API-KEY': self.api_key},
                files=attachments
            )
        else:
            response = requests.post(
                self.notif_service_url,
                headers={
                    'Content-Type': 'application/json',
                    'API-KEY': self.api_key,
                },
                data=body,
            )

        if response.status_code != requests.codes.ok:
            raise Exception('failed send bulk notif ' + response.text)

    def send(self, notification):
        self.bulk_send([notification])




from .abstract_channel import AbstractChannel


class UserPushNotification(AbstractChannel):

    def __init__(
        self, recipient_email, engagement_type, title, body, object_id, url=None, tag=None, data=None,
        message_template=None, message_entity=None
    ):
        self.set_recipient_email(recipient_email)
        self.set_engagement_type(engagement_type)
        self.set_title(title)
        self.set_body(body)
        self.set_object_id(object_id)

        if url is not None:
            self.set_url(url)
        if tag is not None:
            self.set_tag(tag)
        if data is not None:
            self.set_data(data)
        if message_template is not None:
            self.set_message_template(message_template)
        if message_entity is not None:
            self.set_message_entity(message_entity)

    def set_recipient_email(self, email):
        self.email = email

    def set_engagement_type(self, engagement_type):
        self.engagement_type = engagement_type

    def set_title(self, title):
        self.title = title

    def set_body(self, body):
        self.body = body

    def set_object_id(self, object_id):
        self.object_id = object_id

    # optional
    def set_url(self, url):
        self.url = url

    # optional
    def set_tag(self, tag):
        self.tag = tag

    # optional
    def set_data(self, data):
        self.data = data

    # optional
    def set_message_template(self, message_template):
        self.message_template = message_template

    # optional
    def set_message_entity(self, message_entity):
        self.message_entity = message_entity

    def json(self):
        payload = dict()
        payload['recipientEmail'] = self.email
        payload['engagementType'] = self.engagement_type
        payload['title'] = self.title
        payload['body'] = self.body
        payload['objectId'] = self.object_id

        # optional payload
        url = getattr(self, 'url', None)
        tag = getattr(self, 'tag', None)
        data = getattr(self, 'data', None)
        message_template = getattr(self, 'message_template', None)
        message_entity = getattr(self, 'message_entity', None)

        if url is not None:
            payload['url'] = url
        if tag is not None:
            payload['tag'] = tag
        if data is not None:
            payload['data'] = data
        if message_template is not None:
            payload['messageTemplate'] = message_template
        if message_entity is not None:
            payload['messageEntity'] = message_entity

        return payload

    def notif_type(self):
        return 'USER_PUSH_NOTIFICATION'

    def attachments(self):
        return []



from setuptools import setup

setup(
    name='p1-notif',
    version='0.2',
    description='Notification SDK for Dekoruma\'s internal use',
    url='https://github.com/Dekoruma/p1-notif',
    author='Jauhar Arifin',
    author_email='jauhararifin10@gmail.com',
    license='MIT',
    packages=['notif', 'notif.channels'],
    install_requires=['future', 'requests'],
    zip_safe=False
)



from .abstract_channel import AbstractChannel


class WhatsappTemplateMessageSender(AbstractChannel):

    def __init__(self, phone_number, template_id, wa_params, name, provider_account_name=None):
        self.set_phone_number(phone_number)
        self.set_template_id(template_id)
        self.set_wa_params(wa_params)
        self.set_name(name)
        self.set_provider_account_name(provider_account_name)

    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    def set_template_id(self, template_id):
        self.template_id = template_id

    def set_wa_params(self, wa_params):
        self.wa_params = wa_params

    def set_name(self, name):
        self.name = name

    def set_provider_account_name(self, provider_account_name):
        self.provider_account_name = provider_account_name

    def notif_type(self):
        return 'SEND_TEMPLATE_MESSAGE'

    def json(self):
        return {
            'phone_number': self.phone_number,
            'template_id': self.template_id,
            'wa_params': self.wa_params,
            'name': self.name,
            'provider_account_name': self.provider_account_name
        }

    def attachments(self):
        return []


class WhatsappTemplateMessageSenderV3(AbstractChannel):

    def __init__(self, phone_number, template_name, wa_params):
        self.set_phone_number(phone_number)
        self.set_template_name(template_name)
        self.set_wa_params(wa_params)

    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    def set_template_name(self, template_name):
        self.template_name = template_name

    def set_wa_params(self, wa_params):
        self.wa_params = wa_params

    def notif_type(self):
        return 'SEND_TEMPLATE_MESSAGE_V3'

    def json(self):
        return {
            'phone_number': self.phone_number,
            'template_name': self.template_name,
            'wa_params': self.wa_params,
        }

    def attachments(self):
        return []



from abc import ABCMeta, abstractmethod


class AbstractChannel(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def notif_type():
        raise NotImplementedError()

    @abstractmethod
    def json(self):
        raise NotImplementedError()

    @abstractmethod
    def attachments(self):
        raise NotImplementedError()



from .abstract_channel import AbstractChannel

try:
    basestring
except NameError:
    basestring = str


class Email(AbstractChannel):

    def __init__(self, subject, sender, recipients):
        self.set_subject(subject)
        self.set_sender(sender)
        self.set_recipients(recipients)

        self.content_subtype = 'plain'
        self.recipients_cc = []
        self.recipients_bcc = []
        self.email_attachments = []

    def set_subject(self, subject):
        self.subject = subject

    def set_sender(self, sender):
        self.sender = sender

    def set_recipients(self, recipients):
        if isinstance(recipients, basestring):
            recipients = [recipients]
        self.recipients = list(recipients)

    def add_recipients(self, recipients):
        if isinstance(recipients, basestring):
            recipients = [recipients]
        self.recipients.extend(list(recipients))

    def set_recipients_cc(self, recipients_cc):
        if isinstance(recipients_cc, basestring):
            recipients_cc = [recipients_cc]
        self.recipients_cc = list(recipients_cc)

    def add_recipients_cc(self, recipients_cc):
        if isinstance(recipients_cc, basestring):
            recipients_cc = [recipients_cc]
        self.recipients_cc.extend(list(recipients_cc))

    def set_recipients_bcc(self, recipients_bcc):
        if isinstance(recipients_bcc, basestring):
            recipients_bcc = [recipients_bcc]
        self.recipients_bcc = list(recipients_bcc)

    def add_recipients_bcc(self, recipients_bcc):
        if isinstance(recipients_bcc, basestring):
            recipients_bcc = [recipients_bcc]
        self.recipients_bcc.extend(list(recipients_bcc))

    def set_html_message(self, html_message):
        self.message = html_message
        self.set_content_subtype('html')

    def set_text_message(self, message):
        self.message = message
        self.set_content_subtype('plain')

    def set_message(self, message, content_subtype):
        self.message = message
        self.set_content_subtype(content_subtype)

    def set_content_subtype(self, content_subtype):
        self.content_type = content_subtype

    def add_attachment(self, filename, content, content_type):
        attachment = (filename, content, content_type)
        self.email_attachments.append(attachment)

    def notif_type(self):
        return 'EMAIL_NOTIFICATION'

    def json(self):
        return {
            'subject': self.subject,
            'body': self.message,
            'sender': self.sender,
            'recipient': self.recipients,
            'cc': self.recipients_cc,
            'bcc': self.recipients_bcc,
            'contentSubtype': self.content_type
        }

    def attachments(self):
        return self.email_attachments
